//#include <qlcdnumber.h>
#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kmessagebox.h>

#include "kload.h"
#include <qregexp.h>
#include "systeminfo.h"
#include "dlgsettings.h"
#include <qlabel.h>
#include <qtimer.h>
#include <qstringlist.h>
#include <qcursor.h>

kload::kload(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
	: KPanelApplet(configFile, type, actions, parent, name)
{
	//QLCDNumber *w = new QLCDNumber(this);
	QLabel *w = new QLabel(this);
	w->setText("");
	w->setLineWidth(1);
	//w->setFrameShape(Box);
	//w->setFrameShadow(Plain);
	//w->setPaletteBackgroundColor(QColor(255,0,0));
	//w->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed, true);
	//w->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred, FALSE);
	QTimer *t = new QTimer(this);
	connect(t, SIGNAL(timeout()), SLOT(sysUpdate()));
	t->start(5000, FALSE);

	info = new QStringList();
	m_ItemsMenu = new KPopupMenu(0x0, "items_menu");
	connect(m_ItemsMenu, SIGNAL(activated(int)), this, SLOT(refresh_info(int)));
	//updateItemsMenu();

	mainView = w;
	mainView->show();
}


kload::~kload()
{
	delete mainView;
	delete m_ItemsMenu;
	delete info;
}


void kload::about()
{
	KMessageBox::information(0, i18n("This is an about box"));
}


void kload::help()
{
	KMessageBox::information(0, i18n("Left click to see next item.\nShift Left click to see previous one.\nRight click for more options."));
}


void kload::preferences()
{
	dlgSettings *a;
	a = new dlgSettings();
	connect(a, SIGNAL(configChanged()), this, SLOT(updateConfig()));
	a->show();
}

void kload::updateConfig()
{
	conf.update();
	//updateItemsMenu();
	//KMessageBox::information(0, i18n("changes saved."));
}

#include <kiconloader.h>
#include <khelpmenu.h>
void kload::updateItemsMenu()
{
	m_ItemsMenu->clear();
	m_ItemsMenu->insertTitle(/*SmallIcon("cdaudio_unmount"), */i18n("kload"));
	for (unsigned int i = 0; i < info->count(); i++)
	{
		m_ItemsMenu->insertItem(SmallIcon("player_play"), (*info)[i], i);
	}
	m_ItemsMenu->insertSeparator();
	m_ItemsMenu->insertItem(SmallIcon("configure"), i18n("&Configure"), 1001);
	m_ItemsMenu->insertItem(SmallIcon("khelpcenter"), i18n("Quick &Help"), 1002);
	KHelpMenu *help = new KHelpMenu( this, KGlobal::instance()->aboutData(), false);
	m_ItemsMenu->insertItem(i18n( "&Help" ), help->menu(), 1003);
	//m_ItemsMenu->insertSeparator();
}

#include <kdebug.h>
int kload::widthForHeight(int /*height*/) const
{
	int w = mainView->fontMetrics().width(mainView->text() + "   ");
	mainView->setFixedWidth(w);
	return w;
}

int kload::heightForWidth(int /*width*/) const
{
	return fontMetrics().lineSpacing();
	//return mainView->sizeHint().height();
}

void kload::mouseReleaseEvent (QMouseEvent *e)
{
	if (e->button() & RightButton)
	{
		updateItemsMenu();
		showPopupMenu(m_ItemsMenu);
	}
	else
	{/*KMessageBox::information(0,
		QString("buttons: ") + QString::number(e->button(), 2) +
		QString(", states: ") + QString::number(e->state(), 2) +
		QString(", Shift: ") + QString::number(ShiftButton, 2) +
		QString(", Left: ") + QString::number(LeftButton, 2) +
		QString(", Right: ") + QString::number(RightButton, 2)
		);*/
		unsigned int tmp_current = conf.Current;
		if (e->state() & ShiftButton /*RightButton*/)
		{
			//KMessageBox::information(0, i18n("SHIFT..."));
			tmp_current--;
			if (tmp_current > (info->count() - 1))
				tmp_current = info->count() - 1;
			conf.saveCurrent(tmp_current);
			refresh_info();
		}
		else
		{
			tmp_current++;
			if (tmp_current >= info->count())
				tmp_current = 0;
			conf.saveCurrent(tmp_current);
			refresh_info();
		}
		//updateLayout();
	}
}

void kload::refresh_tooltip()
{
	QString data = "";
	for (unsigned int i = 0; i < info->count(); i++)
	{
		data += "\n" + (*info)[i];
	}
}

void kload::refresh_info()
{
	mainView->setText((*info)[conf.Current]);
	widthForHeight(0);
}

void kload::refresh_info(int id)
{
	if (id >= 0 && (unsigned int)id < info->count())
	{
		conf.saveCurrent(id);
		refresh_info();
	}
	else if (id == 1001)
	{
		preferences();
	}
	else if (id == 1002)
	{
		help();
	}
/*	elseif (id == 1003)
	{
		about();
	}*/
	//else { KMessageBox::information(0, QString("hehe") + id); }
}

#include <kwin.h>
void kload::showPopupMenu(KPopupMenu *menu)
{
	ASSERT(menu != 0L);
	menu->move(-1000,-1000);
	menu->show();
	menu->hide();
	KWin::WindowInfo i = KWin::windowInfo(winId());
	if (i.valid(true))
	{
		QRect g = i.geometry();
		if (g.x() > QApplication::desktop()->width()/2
		&& g.y() + menu->height() > QApplication::desktop()->height())
		{
			menu->popup(QPoint(g.x(), g.y() - menu->height()));
		}
		else
		{
			menu->popup(QPoint(g.x() /*+ (width()/3)*/, g.y() + height()));
			//menu->exec(mapToGlobal(QPoint(width()/2, height()/2)));
		}
	}
	else
	{
		QPoint g = QCursor::pos();
		if (menu->height() < g.y())
			menu->popup(QPoint(g.x(), g.y() - menu->height()));
		else
			menu->popup(QPoint(g.x(), g.y()));
	}
}

QString secs2uptime(long s)
{
	/*
	long secs_p_d = 60 * 60 * 24;
	long r = s % secs_p_d;
	QString result = QString::number((s - r) / secs_p_d);
	result += " days ";
	QDateTime t; t.setTime_t(s);
	result += t.toString("hh:mm:ss");
	return result;
	*/

	long uptimeDays, uptimeHours, uptimeMins, uptimeSecs;

	uptimeHours = s / 3600;
	uptimeDays = uptimeHours / 24;
	uptimeMins = (s - uptimeHours * 3600) / 60;
	uptimeSecs = s % 60;
	uptimeHours -= (uptimeDays * 24);
	return QString::number(uptimeDays) + " days " + QString::number(uptimeHours) + (uptimeMins < 10 ? ":0" : ":")+ QString::number(uptimeMins); // + ":" + QString::number(uptimeSecs);
}

double b2k(double b)
{
	return b / 1024;
}

double b2m(double b)
{
	return b / 1024 / 1024;
}

// void msgbox(QString s)
// {
// }

QString n2s(double n)
{
	return QString::number(n, 'f', 1);
}

void kload::addInfo(QString slNewInfo)
{
	info->append(slNewInfo.simplifyWhiteSpace());
}

#include <qstringlist.h>
void kload::sysUpdate()
{
	const System &system = System::self();
	info->clear();
	if (conf.Uptime)
	{
		addInfo(secs2uptime(system.uptime()));
	}
	if (conf.CPU)
	{
		addInfo(QString("sorry, feature (CPU Usage) not fully implemented."));
		FILE *file = fopen("/proc/stat", "r");
		char buffer[200];
		double user = 0, system = 0, nice = 0, idle = 0;
		if (file)
		{
			bool done = false;
			while (fgets(buffer, sizeof(buffer), file) && !done)
			{
/*
				// sscanf suxx is not it ?
				QString net_dev_format = "%lu %lu %lu %lu";
				err_code = sscanf(buffer+5, net_dev_format,
					&user, &system, &nice, &idle);
				if (err_code != 0)
				{
					iface = QString(buffer).left(5);
						kdDebug() << "yy" << endl;
				}
				else
				{
					// kernel 2.6
					net_dev_format += " %lu %lu %lu";
					err_code = sscanf(buffer+5, net_dev_format,
						&user, &system, &nice, &idle, &wa, &hi, &si);
					if (err_code != 0)
					{
						iface = QString(buffer).left(5);
						kdDebug() << "xx" << endl;
					}
				}*/
				QStringList sl = QStringList::split(" ", buffer);
				if (sl[0] == "cpu")
				{
					user = sl[1].toULong();
					system = sl[2].toULong();
					nice = sl[3].toULong();
					idle = sl[4].toULong();
					double used = b2k(user) + b2k(system);
					double total = used + b2k(idle) + b2k(nice);
					kdDebug()
						<< endl
						<< " user " << user
						<< " sys " << system
						<< " nice " << nice
						<< " idle " << idle
						<< endl
						<< " user " << b2k(user)
						<< " sys " << b2k(system)
						<< " nice " << b2k(nice)
						<< " idle " << b2k(idle)
						<< endl
						<< " used " << used
						<< " total " << total
						<< " % " << (used / total)
						<< " % " << n2s(used / total)
						<< endl;
					addInfo(QString("CPU ") + n2s(used / total * 100) + "%");
					//done = true;
				}
			}
			fclose(file);
		}
	}
	if (conf.Load)
	{
		addInfo(QString::number(system.loadAverage(1)) + ", " + QString::number(system.loadAverage(2)) + ", " + QString::number(system.loadAverage(3)));
	}

	if (!conf.RamNone)
	{
		double total = system.totalRam();
		double free = system.freeRam();
		double buffer = system.bufferRam();
		double cache = system.cacheRam();
		//double used = system.usedRam();
		double allFree = free + buffer + cache;
		double allUsed = total - allFree;
		/*kdDebug()	<< "total " << total
					<< " free " << free
					<< " buffer " << buffer
					<< " cache " << cache
					//<< " used " << used
					<< " allUsed / total " << (allUsed / total)
					<< " b2m " << b2m(allUsed / total)
					<< " n2s " << n2s(b2m(allUsed / total))
					<< endl;*/

		if (conf.RamStd)
		{
			QString memory = "Mem %U / %t";
			//memory.replace(QRegExp("%u"), n2s(b2m(used)));
			memory.replace(QRegExp("%t"), n2s(b2m(total)));
			//memory.replace(QRegExp("%F"), n2s(b2m(allFree)));
			memory.replace(QRegExp("%U"), n2s(b2m(allUsed)));
			addInfo(memory);
		}
		else if (conf.RamP100)
		{
			QString memory = "Mem %u%";
			memory.replace(QRegExp("%u"), n2s(allUsed / total * 100));
			//KMessageBox::information(0, QString::number(allUsed) + " " + QString::number(total) + " " + QString::number((allUsed / total)) + " " + QString::number(b2m(allUsed / total)));
			addInfo(memory);
		}
	}

	if (conf.SwapStd)
	{
		QString swap = "Swap %u / %t";
		unsigned long total = system.totalSwap();
		unsigned long swapUsed = system.usedSwap();
		swap.replace(QRegExp("%u"), n2s(b2m(swapUsed)));
		swap.replace(QRegExp("%t"), n2s(b2m(total)));
		addInfo(swap);
	}
	else if (conf.SwapP100)
	{
		QString swap = "Swap %u%";
		swap.replace(QRegExp("%u"), n2s(((double)system.usedSwap() / (double)system.totalSwap() * 100)));
		addInfo(swap);
	}


	if (conf.Net)
	{
		FILE *file = fopen("/proc/net/dev", "r");
		char buffer[200];
		int err_code;
		unsigned long rx, tx;
		unsigned long old_rx, old_tx;
		QString iface;
		if (file)
		{
			while (fgets(buffer, sizeof(buffer), file))
			{
				QString net_dev_format = "%lu %*u %*u %*u %*u %*u %*u %*u %lu";
				err_code = sscanf(buffer+7, net_dev_format, &rx, &tx);
				if (err_code != 0)
				{
					iface = QString(buffer).left(6);
					if (conf.NetExclusion.simplifyWhiteSpace().contains(iface.simplifyWhiteSpace()) == 0)
					{
						//KMessageBox::information(0, iface + "  " + conf.NetExclusion);
						old_rx = 0;
						if (net_vals[iface + "rx"])
							old_rx = (unsigned long)(net_vals[iface + "rx"]);

						old_tx = 0;
						if (net_vals[iface + "tx"])
							old_tx = (unsigned long)(net_vals[iface + "tx"]);
						if (conf.NetTotal)
							addInfo(iface + " " + n2s(b2m(rx)) + " - " + n2s(b2m(tx)) + " M");
						if (conf.NetSpeed)
							addInfo(iface + " " + n2s(b2k(rx - old_rx) / 5) + " - " + n2s(b2k(tx - old_tx) / 5) + " K/s");
 						/*kdDebug() << iface + " t " + QString::number(rx)
							<< "  :  " << QString::number(old_rx) << endl;
						kdDebug() << "backuping rx " << rx << " for "
							<< iface << endl;
						*/
						net_vals.remove(iface + "rx");
						net_vals.remove(iface + "tx");
						net_vals.insert(iface + "rx", (void*)rx);
						net_vals.insert(iface + "tx", (void*)tx);
					}
				}
			}
			fclose(file);
		}
	}
	refresh_info();
}

extern "C"
{
	KPanelApplet* init( QWidget *parent, const QString configFile)
	{
		KGlobal::locale()->insertCatalogue("kload");
		return new kload(configFile, KPanelApplet::Stretch, KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences, parent, "kload");
	}
}
